@@ -6,10 +6,11 @@ import datetime
 import os
 
 from django.contrib.admin.views.decorators import staff_member_required
-from django.http import HttpResponse
+from django.http import HttpResponse, Http404
 from django.shortcuts import get_object_or_404
 from django.views.decorators.http import require_http_methods
 from django.views.decorators.csrf import csrf_exempt
+from django.views.generic.detail import DetailView
 
 from .conf import settings
 from .models import Video, Job
@@ -30,16 +31,7 @@ def encode(request, video_id):
     video = get_object_or_404(Video, pk=video_id)
 
     job = Job.objects.start(video=video)
-    fmt = {
-        "base_url": "https://app.zencoder.com/api/v2/jobs/{}/progress".format(job.job_id),
-        "api_key": settings.ZENCODER_API_KEY
-    }
-
-    data = {
-        "json": "{base_url}.json?api_key={api_key}".format(**fmt),
-        "xml": "{base_url}.xml?api_key={api_key}".format(**fmt),
-        "js": "{base_url}.js?api_key={api_key}".format(**fmt),
-    }
+    data = job.encode_status_endpoints()
 
     return HttpResponse(json.dumps(data), content_type="application/json")
 
@@ -89,14 +81,18 @@ def video(request, video_id=None):
     upload_endpoint = "https://{}.s3.amazonaws.com".format(settings.VIDEO_ENCODING_BUCKET)
 
     status = "Not started"
+    encode_status_endpoints = None
     if video.job_set.count() > 0:
-        status = video.job_set.all()[0].get_status_display()
+        job = video.job_set.all()[0]
+        status = job.get_status_display()
+        encode_status_endpoints = job.encode_status_endpoints()
 
     contents = {
         "id": video.id,
         "path": video.input,
         "key": key,
         "status": status,
+        "encode_status_endpoints": encode_status_endpoints,
         "upload_endpoint": upload_endpoint,
         'AWSAccessKeyId': settings.AWS_ACCESS_KEY_ID,
         'acl': 'private',
@@ -105,3 +101,50 @@ def video(request, video_id=None):
         'signature': base64.b64encode(signature).decode("utf-8")
     }
     return HttpResponse(json.dumps(contents), status=status_code, content_type="application/json")
+
+
+class VideoEmbedView(DetailView):
+    model = Video
+    context_object_name = "video"
+
+    def get_object(self, queryset=None):
+        if queryset is None:
+            queryset = self.get_queryset()
+        pk = self.request.GET.get("id", None)
+        try:
+            obj = queryset.get(pk=pk)
+        except queryset.model.DoesNotExist:
+            raise Http404("No Video found")
+        return obj
+
+    def get_template_names(self):
+        names = ["zencoder/embed/video.html", "zencoder/embed/default.html"]
+        if self.object.job_set.count() > 0:
+            job = self.object.job_set.all()[0]
+            if job.status == Job.IN_PROGRESS:
+                names.insert(0, "zencoder/embed/encoding.html")
+            elif job.status != Job.COMPLETE:
+                names.insert(0, "zencoder/embed/failed.html")
+        return names
+
+    def render_to_response(self, context, **response_kwargs):
+        if self.object.sources.count() == 0:
+            # If there aren't any sources, this is a bad video
+            response_kwargs["status"] = 500
+
+        if self.object.job_set.count() > 0:
+            job = self.object.job_set.all()[0]
+
+            if job.status == Job.IN_PROGRESS:
+                # If the video is still encoding, we'll make it a 202
+                response_kwargs["status"] = 202
+            elif job.status != Job.COMPLETE:
+                response_kwargs["status"] = 500
+
+        response = super(VideoEmbedView, self).render_to_response(context, **response_kwargs)
+        if response.status_code != 200:
+            response["Cache-Control"] = "no-cache, no-store, must-revalidate"
+            response["Pragma"] = "no-cache"
+            response["Expires"] = "0"
+        return response
+embed = VideoEmbedView.as_view()
